#include "TArtRecoFragment.hh" 
#include "TArtFragment.hh"
#include "TArtDCTrack.hh"
#include "TArtHODPla.hh"
#include "TArtPlastic.hh"

#include "TArtCore.hh" 
#include "TArtUtil.hh" 
#include "TArtStoreManager.hh"
#include "TArtCalcGlobalTrack.hh"

#include "DCConfMan.hh"
#include "FieldMan.hh"

#include <TClonesArray.h>

#include <iostream>
#include <cstdlib>
#include <fstream>

//__________________________________________________________
TArtRecoFragment::TArtRecoFragment() : fFragmentArray(NULL), 
				       hod_array(NULL), 
				       pla_tzero(NULL),
				       fMatReady(false), 
				       fRKtraceReady(true)
{

  // default value from samurai dayone //////
  // do not edit following parameter!! //////
  // use SetCenterBrho function. Aki //////// 
  center_brho = 7.2751;
  l_pla2target = 2824.; // mm
  tof_offset = 146.5; // ns
  ////////////////////////

  clight = 299.792458; // mm/nsec
  mnucleon = 931.49432; // MeV

  TArtCore::Info(__FILE__,"Creating the Reco Fragment objects...");
  sman = TArtStoreManager::Instance();

  fFragmentArray = new TClonesArray("TArtFragment",4);
  fFragmentArray->SetName("SAMURAIFragment");
  fFragmentArray->SetOwner();
  sman->AddDataContainer(fFragmentArray);

  TString mat0thfile = std::getenv("SAMURAI_MATRIX0TH_FILE");
  TString mat1stfile = std::getenv("SAMURAI_MATRIX1ST_FILE");
  TString magfieldfile = std::getenv("SAMURAI_BFIELD_FILE");
  TString dcgeomfile = std::getenv("SAMURAI_DCGEOM_FILE");

  if(!(DCConfMan::GetInstance().Initialize(dcgeomfile.Data())))
    fRKtraceReady = false;
  if(!(FieldMan::GetInstance().Initialize(magfieldfile.Data(),1.0)))
    fRKtraceReady = false;

  std::ifstream mat0thin(mat0thfile.Data());
  std::ifstream mat1stin(mat1stfile.Data());

  if(mat0thin.is_open() && mat1stin.is_open()){

    mat0.ResizeTo(2,1);
    mat0thin >> mat0(0,0) >> mat0(1,0);

    Double_t mat[3][3];
    mat1stin >> mat[0][0] >>  mat[0][1] >> mat[0][2];
    mat1stin >> mat[1][0] >>  mat[1][1] >> mat[1][2];
    mat1stin >> mat[2][0] >>  mat[2][1] >> mat[2][2];

    mat1.ResizeTo(2,1);
    mat1(0,0) = mat[0][0];
    mat1(1,0) = mat[1][0];
    mat2.ResizeTo(2,2);
    mat2(0,0) = mat[0][1];
    mat2(1,0) = mat[1][1];
    mat2(0,1) = mat[0][2];
    mat2(1,1) = mat[1][2];
    inv_mat2.ResizeTo(2,2);
    inv_mat2=mat2.Invert();

    fMatReady = true;

  }
  else if(!(mat0thin.is_open())){
    TArtCore::Error(__FILE__,"fail to read 0th mat file: %s",mat0thfile.Data());
    fMatReady = false;
  }
  else if(!(mat1stin.is_open())){
    TArtCore::Error(__FILE__,"fail to read 1st mat file: %s",mat1stfile.Data());
    fMatReady = false;
  }

  mat0thin.close();
  mat1stin.close();

  if(fRKtraceReady){
    TClonesArray * pla_array = (TClonesArray *)sman->FindDataContainer("BigRIPSPlastic");
    pla_tzero = (TArtPlastic *)TArtUtil::FindDataObject(pla_array,"F13pl-2");
    hod_array = (TClonesArray *)sman->FindDataContainer("SAMURAIHODPla");
  }

}

//__________________________________________________________
TArtRecoFragment::~TArtRecoFragment()  {
  ClearData();
}

//__________________________________________________________
void TArtRecoFragment::ClearData()   {
  fFragmentArray->Clear();
  fReconstructed = false;
  return;
}

//__________________________________________________________
void TArtRecoFragment::ReconstructData() { 

  if((!fMatReady) && (!fRKtraceReady))return;

  TClonesArray * fdc1trks = (TClonesArray *)sman->FindDataContainer("SAMURAIFDC1Track");
  TClonesArray * fdc2trks = (TClonesArray *)sman->FindDataContainer("SAMURAIFDC2Track");

  if(!(fdc1trks && fdc2trks)) return;

  Int_t ntr_fdc1 = fdc1trks->GetEntriesFast();
  Int_t ntr_fdc2 = fdc2trks->GetEntriesFast();
  if(!(ntr_fdc1>0 && ntr_fdc2>0)){
    fReconstructed = true;
    return;
  }

  // as of 2013/10/10 one charged particle is assumed

  TArtDCTrack * fdc1tr = (TArtDCTrack*)fdc1trks->At(0); // chi2 minimum track
  TArtDCTrack * fdc2tr = (TArtDCTrack*)fdc2trks->At(0); // chi2 minimum track
  TArtCore::Debug(__FILE__,"fdcnhit: %d, %d",fdc1tr->GetNumHitLayer(),fdc2tr->GetNumHitLayer());
  TArtCore::Debug(__FILE__,"fdcchi2: %f, %f",fdc1tr->GetChi2(),fdc2tr->GetChi2());
  TArtCore::Debug(__FILE__,"fdc1x: %f, fdc1a: %f",fdc1tr->GetPosition(0),fdc1tr->GetAngle(0));
  TArtCore::Debug(__FILE__,"fdc2x: %f, fdc2a: %f",fdc2tr->GetPosition(0),fdc2tr->GetAngle(0));

  new ((*fFragmentArray)[0]) TArtFragment();
  TArtFragment * frag = (TArtFragment *)fFragmentArray->At(0);

  if(fRKtraceReady) {
    //  TArtCalcGlobalTrack gtr(fdc1tr,fdc2tr,center_brho*(1.+rvec(1,0))*0.3);
    TArtCalcGlobalTrack gtr(fdc1tr,fdc2tr,center_brho*0.3);
    gtr.doFit();
    frag->SetRKtraceStatus(gtr.Status());
    frag->SetChi2(gtr.chisqr());
    frag->SetNHit(gtr.GetNHits());
    frag->SetNIteration(gtr.Niteration());
    frag->SetPrimaryPosition(gtr.PrimaryPosition());
    frag->SetPrimaryMomentum(gtr.PrimaryMomentum());
    Double_t toflength = gtr.PathLengthToTOF() + l_pla2target; // mm;
    frag->SetPathLengthToHOD(toflength); // target - HOD path length
    for(int i=0;i<100;i++) frag->SetDiff(i,gtr.diff(i));

    if(hod_array && pla_tzero){
      if(hod_array->GetEntries()>0){
	TArtHODPla *pla = (TArtHODPla *)hod_array->At(0); // pickup qmax-hod
	Double_t tzero = pla_tzero->GetTime();
	frag->SetTzero(tzero);
	frag->SetTzeroQ(pla_tzero->GetQAveRaw());
        Double_t hodt = pla->GetTime();
	frag->SetHODTime(hodt);
	frag->SetHODQ(pla->GetQAveCal());
	Double_t tof = hodt-tzero+tof_offset;
	//TArtCore::Info(__FILE__,"tzero: %f , hodt: %f",tzero,hodt);
	frag->SetTOF(tof);
	Double_t beta = toflength/tof/clight;
	frag->SetBeta(beta);
	Double_t gamma = 1/TMath::Sqrt(1-beta*beta);
	Double_t brho = frag->GetBrho();
	Double_t aoq = brho * clight / mnucleon / beta / gamma;
	frag->SetAoQ(aoq);
	//TArtCore::Info(__FILE__,"beta: %f , aoq: %f",beta,aoq);
      }
    }
    else {
      TArtCore::Info(__FILE__,"cannot find pla %x %x",hod_array,pla_tzero);
    }
  }
  else if(fMatReady) {
    TMatrixD outxvec(2,1);
    outxvec(0,0) = fdc2tr->GetPosition(0);
    outxvec(1,0) = fdc2tr->GetAngle(0);

    TMatrixD rvec = inv_mat2 * (outxvec - fdc1tr->GetPosition(0) * mat1);
    frag->SetDelta(rvec(1,0));
    TArtCore::Debug(__FILE__,"Delta: %f",rvec(1,0));
  }

  fReconstructed = true;
  return;

}

ClassImp(TArtRecoFragment)
